@@ -1,12 +1,26 @@
-MOPT <-
-function(LB,UB,P,lambda,delta,r=10,grid=0.01,epsilon=.001,epsilon_w=10^-6){
-      t1=P[2]
-	t2=-P[4]
-	t3=P[4]*log(P[3])
-	t4=P[1]
-	T=c(t1,t2,t3,t4)
-      X=c(LB,LB+(UB-LB)/3,LB+2*(UB-LB)/3,UB)
-      W=rep(1/4,3)
+MOPT<-function(LB,UB,P,lambda,delta,r=10,grid=0.01,epsilon=.001,epsilon_w=10^-6,verbose=FALSE){
+
+	np=length(P)
+	if (np==2)
+	{t1=-P[2]
+	 t2=P[2]*log(P[1])
+	 T=c(1,t1,t2,0)} else if(np==3)
+	 {t1=P[1]
+	  t2=-P[3]
+	  t3=P[3]*log(P[2])
+	  T=c(t1,t2,t3,0)} else
+      {t1=P[2]
+	   t2=-P[4]
+	   t3=P[4]*log(P[3])
+	   t4=P[1]
+	   T=c(t1,t2,t3,t4)}
+	   
+      X1=runif(2,LB,UB)
+      X2=runif(3,LB,UB)
+      X3=c(LB,LB+(UB-LB)/3,LB+2*(UB-LB)/3,UB)
+      if (np==2) {X=X1} else if (np==3) {X=X2} else {X=X3}
+      wX=length(X)
+      W=rep(1/wX,wX-1)
       n=1
       p=1
 	e1=epsilon_w
@@ -27,15 +41,30 @@ function(LB,UB,P,lambda,delta,r=10,grid=0.01,epsilon=.001,epsilon_w=10^-6){
               s$v[, nz] %*% (t(s$u[, nz])/s$d[nz])
           else X, dimnames = dnx[2:1])
       }
-      
-      infor <- function(T, X) {
+      infor2 <- function(T, X) {
+          f = matrix(c(exp(1/2*(T[2] * X + T[3]))/(1 + exp(T[2] * X + T[3])),X*exp(1/2*(T[2] * X + T[3]))/
+          (1 + exp(T[2] *X + T[3]))), nrow = 2, ncol = 1, byrow = F)
+          f %*% t(f)
+      }     
+      infor3 <- function(T, X) {
+          f = matrix(c(1/(1 + exp(T[2] * X + T[3])), (-T[1] * X * 
+              exp(T[2] * X + T[3]))/(1 + exp(T[2] * X + T[3]))^2, 
+              (-T[1] * exp(T[2] * X + T[3]))/(1 + exp(T[2] * X + 
+                T[3]))^2), nrow = 3, ncol = 1, byrow = F)
+          f %*% t(f)
+      }
+      infor4 <- function(T, X) {
           f = matrix(c(1/(1 + exp(T[2] * X + T[3])), (-T[1] * X * 
               exp(T[2] * X + T[3]))/(1 + exp(T[2] * X + T[3]))^2, 
               (-T[1] * exp(T[2] * X + T[3]))/(1 + exp(T[2] * X + 
                 T[3]))^2, 1), nrow = 4, ncol = 1, byrow = F)
           f %*% t(f)
       }
-
+ 
+      infor <- function(T,X){
+      	if(np==2) {infor2(T,X)} else if (np==3)
+      	{infor3(T,X)} else {infor4(T,X)}}
+     
       upinfor <- function(W, T, X) {
           k = length(X)
           last_infor = infor(T, X[k])
@@ -54,14 +83,22 @@ function(LB,UB,P,lambda,delta,r=10,grid=0.01,epsilon=.001,epsilon_w=10^-6){
           matrix(c(-1/((T[1]+dt)*T[2]),(-log(-dt/(T[1]+dt))+T[3])/T[2]^2,-1/T[2],0),nrow=4,ncol=1,byrow=F)
       }
 
-      g1<-function(T) {
+      tg1<-function(T) {
           matrix(c(0,-T[3]/(T[2])^2,-1/T[2],0),nrow=4,ncol=1,byrow=F)
       } 
+      g1<-function(T){tempT=tg1(T)
+      	    if (np==2) {matrix(c(tempT[2],tempT[3]),nrow=2,ncol=1,byrow=F)}else if
+      	    (np==3){matrix(c(0,tempT[2],tempT[3]),nrow=3,ncol=1,byrow=F)}else
+      	    {tempT}}
 
-      g2<-function(T, dt) {
+      tg2<-function(T, dt) {
           if(T[2]<0) g21(T,dt) else g22(T,dt)
       }    
-
+	    g2<-function(T,dt){tempT=tg2(T,dt)
+      	    if (np==2) {matrix(c(tempT[2],tempT[3]),nrow=2,ncol=1,byrow=F)}else if
+      	    (np==3){matrix(c(tempT[1],tempT[2],tempT[3]),nrow=3,ncol=1,byrow=F)}else
+      	    {tempT}}
+      	    
       d1<-function(T,X,XL,inv) {
           sum(diag(inv%*%(infor(T,X)-infor(T,XL))))
       }
@@ -102,11 +139,11 @@ function(LB,UB,P,lambda,delta,r=10,grid=0.01,epsilon=.001,epsilon_w=10^-6){
           f1=rep(0,p)
           f2=matrix(c(rep(f1,p)),nrow=p,ncol=p,byrow=F)
           for (i in 1:p) {
-               f1[i]=q1*d1(T,X[i],X[k],inv)/4-q2*d2(T,X[i],X[k],inv)-q3*d3(T,X[i],X[k],inv,dt)
+               f1[i]=q1*d1(T,X[i],X[k],inv)/np-q2*d2(T,X[i],X[k],inv)-q3*d3(T,X[i],X[k],inv,dt)
           }
           for(i in 1:p) {
                for(j in 1:p) {
-                     f2[i,j]=q1*dd1(T,X[i],X[j],X[k],inv)/4-q2*dd2(T,X[i],X[j],X[k],inv)-q3*dd3(T,X[i],X[j],X[k],inv,dt)
+                     f2[i,j]=q1*dd1(T,X[i],X[j],X[k],inv)/np-q2*dd2(T,X[i],X[j],X[k],inv)-q3*dd3(T,X[i],X[j],X[k],inv,dt)
                }
           }
           newweight=W-d*(f1%*%ginv(f2))
@@ -153,11 +190,27 @@ function(LB,UB,P,lambda,delta,r=10,grid=0.01,epsilon=.001,epsilon_w=10^-6){
           D
       }
 
-      f=function(T,X) {
-            matrix(c(1/(1+exp(T[2]*X+T[3])),(-T[1]*X*exp(T[2]*X+T[3]))/(1+exp(T[2]*X+T[3]))^2,(-T[1]*exp(T[2]*X+T[3]))/(1+exp(T[2]*X+T[3]))^2,1),nrow=4,ncol=1,byrow=F)
-      }
+
+      f2 <- function(T, X) {
+          matrix(c(exp(1/2*(T[2] * X + T[3]))/(1 + exp(T[2] * X + T[3])),X*exp(1/2*(T[2] * X + T[3]))/
+        (1 + exp(T[2] *X + T[3]))), nrow = 2, ncol = 1, byrow = F)
+          }     
+      f3 <- function(T, X) {
+          matrix(c(1/(1 + exp(T[2] * X + T[3])), (-T[1] * X * 
+              exp(T[2] * X + T[3]))/(1 + exp(T[2] * X + T[3]))^2, 
+              (-T[1] * exp(T[2] * X + T[3]))/(1 + exp(T[2] * X + 
+                T[3]))^2), nrow = 3, ncol = 1, byrow = F)
+          }      
+      f4=function(T,X) {
+            matrix(c(1/(1+exp(T[2]*X+T[3])),(-T[1]*X*exp(T[2]*X+T[3]))/(1+exp(T[2]*X+T[3]))^2,(-T[1]*exp(T[2]*X+T[3]))/(1+exp(T[2]*X+T[3]))			^2,1),nrow=4,ncol=1,byrow=F)
+      	  }
+ 
+      f <- function(T,X){
+      	if(np==2) {f2(T,X)} else if (np==3)
+      	{f3(T,X)} else {f4(T,X)}}
+      	
       ds1=function(T,X,inv) {
-            (t(f(T,X))%*%inv%*%f(T,X))/4
+            (t(f(T,X))%*%inv%*%f(T,X))/np
       }
       ds2=function(T,X,inv) {
             t(f(T,X))%*%inv%*%g1(T)%*%t(g1(T))%*%inv%*%f(T,X)%*%(t(g1(T))%*%inv%*%g1(T))^-1
@@ -190,11 +243,13 @@ function(LB,UB,P,lambda,delta,r=10,grid=0.01,epsilon=.001,epsilon_w=10^-6){
             n=n+1
       }
       r=length(X)
-      X=unique(X[(r-4):r])
+      X=unique(X[(r-np):r])
 	X=sort(X,decreasing=F)
+      if (length(X)==1) X=c(X,runif(1,LB,UB))
       n=1
       p=1
       it=1
+      cat("Computing maximum value of the sensitvity function...","\n")
       while(p>e2) {
             x=seq(LB,UB,gr)
             n1=length(x)
@@ -220,11 +275,12 @@ function(LB,UB,P,lambda,delta,r=10,grid=0.01,epsilon=.001,epsilon_w=10^-6){
             if(abs(newp-p)<.0000001)   newp=10^-20
             if(it>20)   newp=10^-20
             p=newp
+            if(verbose) 
+                  cat("Iteration",it,":",p,"\n")
             it=it+1
-            print(p)
       }
 
-      k=4
+      
       X=D[1,]
       n=length(X)
       W=D[2,1:n-1]
@@ -236,6 +292,11 @@ function(LB,UB,P,lambda,delta,r=10,grid=0.01,epsilon=.001,epsilon_w=10^-6){
             ds[i]=q1*ds1(T,x[i],inv)+q2*ds2(T,x[i],inv)+q3*ds3(T,x[i],inv,dt)
       }
 
-      plot(x,ds,cex=.3,main="Verify the Multi-obj optimal design",ylab="Sensitive function",xlab="Dose levels")
-      D
+      cat("_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _","\n")
+      colnames(D)<-NULL
+      rownames(D)<-c("X","W")
+
+      R=new("PAR",fid="MOPT",LB=LB,UB=UB,grid=grid,ds=ds)
+      return(new("OPT",Par=R,Opt=D))
 }
+
