@@ -1,11 +1,19 @@
-ceff2 <-
-function(weight,P,dose,LB,UB,delta,r=10,grid=0.01,epsilon=.001,epsilon_w=10^-6)
+ceff2<-function(weight,P,dose,LB,UB,delta,r=10,grid=0.01,epsilon=.001,epsilon_w=10^-6)
 { 
-      t1=P[2]
-      t2=-P[4]
-      t3=P[4]*log(P[3])
-      t4=P[1]
-      T=c(t1,t2,t3,t4)
+	np=length(P)
+	if (np==2)
+	{t1=-P[2]
+	 t2=P[2]*log(P[1])
+	 T=c(1,t1,t2,0)} else if(np==3)
+	 {t1=P[1]
+	  t2=-P[3]
+	  t3=P[3]*log(P[2])
+	  T=c(t1,t2,t3,0)} else
+      {t1=P[2]
+	   t2=-P[4]
+	   t3=P[4]*log(P[3])
+	   t4=P[1]
+	   T=c(t1,t2,t3,t4)} 
 	e1=epsilon_w
 	e2=epsilon
 	dt=delta
@@ -29,10 +37,30 @@ function(weight,P,dose,LB,UB,delta,r=10,grid=0.01,epsilon=.001,epsilon_w=10^-6)
       k = length(T)
       nit = nit
      
-      infor <- function(T, X) {
-           f = matrix(c(1/(1 + exp(T[2] * X + T[3])), (-T[1] * X * exp(T[2] * X + T[3]))/(1 + exp(T[2] * X + T[3]))^2, (-T[1] * exp(T[2] * X + T[3]))/(1 + exp(T[2] * X + T[3]))^2, 1), nrow = 4, ncol = 1, byrow = F)
-           f %*% t(f)
+ 	infor2 <- function(T, X) {
+          f = matrix(c(exp(1/2*(T[2] * X + T[3]))/(1 + exp(T[2] * X + T[3])),X*exp(1/2*(T[2] * X + T[3]))/
+          (1 + exp(T[2] *X + T[3]))), nrow = 2, ncol = 1, byrow = F)
+          f %*% t(f)
+      }     
+      infor3 <- function(T, X) {
+          f = matrix(c(1/(1 + exp(T[2] * X + T[3])), (-T[1] * X * 
+              exp(T[2] * X + T[3]))/(1 + exp(T[2] * X + T[3]))^2, 
+              (-T[1] * exp(T[2] * X + T[3]))/(1 + exp(T[2] * X + 
+                T[3]))^2), nrow = 3, ncol = 1, byrow = F)
+          f %*% t(f)
       }
+      infor4 <- function(T, X) {
+          f = matrix(c(1/(1 + exp(T[2] * X + T[3])), (-T[1] * X * 
+              exp(T[2] * X + T[3]))/(1 + exp(T[2] * X + T[3]))^2, 
+              (-T[1] * exp(T[2] * X + T[3]))/(1 + exp(T[2] * X + 
+                T[3]))^2, 1), nrow = 4, ncol = 1, byrow = F)
+          f %*% t(f)
+      }
+ 
+      infor <- function(T,X){
+      	if(np==2) {infor2(T,X)} else if (np==3)
+      	{infor3(T,X)} else {infor4(T,X)}}
+
 
       upinfor <- function(W, T, X) {
            k = length(X)
@@ -43,14 +71,20 @@ function(weight,P,dose,LB,UB,delta,r=10,grid=0.01,epsilon=.001,epsilon_w=10^-6)
            infor
       }
 
-      g1<-function(T,dt) 
-           matrix(c(1/((T[1]-dt)*T[2]),(-log((T[1]-dt)/dt)+T[3])/T[2]^2,-1/T[2],0),nrow=4,ncol=1,byrow=F)
+      g21<-function(T,dt) {
+          matrix(c(1/((T[1]-dt)*T[2]),(-log((T[1]-dt)/dt)+T[3])/T[2]^2,-1/T[2],0),nrow=4,ncol=1,byrow=F)
+      }
 
-      g2<-function(T,dt) 
-           matrix(c(-1/((T[1]+dt)*T[2]),(-log(-dt/(T[1]+dt))+T[3])/T[2]^2,-1/T[2],0),nrow=4,ncol=1,byrow=F)
-
-      g<-function(T,dt) 
-           if(T[2]<0) g1(T,dt) else g2(T,dt)
+      g22<-function(T,dt) {
+          matrix(c(-1/((T[1]+dt)*T[2]),(-log(-dt/(T[1]+dt))+T[3])/T[2]^2,-1/T[2],0),nrow=4,ncol=1,byrow=F)
+      }
+      tg2<-function(T, dt) {
+          if(T[2]<0) g21(T,dt) else g22(T,dt)
+      }    
+	    g<-function(T,dt){tempT=tg2(T,dt)
+      	    if (np==2) {matrix(c(tempT[2],tempT[3]),nrow=2,ncol=1,byrow=F)}else if
+      	    (np==3){matrix(c(tempT[1],tempT[2],tempT[3]),nrow=3,ncol=1,byrow=F)}else
+      	    {tempT}}
 
       d3<-function(T,X,XL,inv,dt) 
            -t(g(T,dt))%*%inv%*%(infor(T,X)-infor(T,XL))%*%inv%*%g(T,dt)%*%(t(g(T,dt))%*%inv%*%g(T,dt))^-1
@@ -85,7 +119,7 @@ function(weight,P,dose,LB,UB,delta,r=10,grid=0.01,epsilon=.001,epsilon_w=10^-6)
            k=length(X)
            W=rep(1/k,k-1)
            while(diff>e1) {
-                d=.2
+                d=1
                 NW=c_weight(W,T,X,d)
                 minW=min(min(NW),1-sum(NW))
                 while(minW<0 & d>.0001) {
@@ -117,14 +151,32 @@ function(weight,P,dose,LB,UB,delta,r=10,grid=0.01,epsilon=.001,epsilon_w=10^-6)
            D
       }
 
-      X=c(LB,LB+(UB-LB)/3,LB+2*(UB-LB)/3,UB)
-      W=rep(1/4,3)
+      X1=runif(2,LB,UB)
+      X2=runif(3,LB,UB)
+      X3=c(LB,LB+(UB-LB)/3,LB+2*(UB-LB)/3,UB)
+      if (np==2) {X=X1} else if (np==3) {X=X2} else {X=X3}
+      wX=length(X)
+      W=rep(1/wX,wX-1)
       n=1
       p=1
    
-      f<-function(T,X) 
-           matrix(c(1/(1+exp(T[2]*X+T[3])),(-T[1]*X*exp(T[2]*X+T[3]))/(1+exp(T[2]*X+T[3]))^2,(-T[1]*exp(T[2]*X+T[3]))/(1+exp(T[2]*X+T[3]))^2,1),nrow=4,ncol=1,byrow=F)
-
+      f2 <- function(T, X) {
+          matrix(c(exp(1/2*(T[2] * X + T[3]))/(1 + exp(T[2] * X + T[3])),X*exp(1/2*(T[2] * X + T[3]))/
+        (1 + exp(T[2] *X + T[3]))), nrow = 2, ncol = 1, byrow = F)
+          }     
+      f3 <- function(T, X) {
+          matrix(c(1/(1 + exp(T[2] * X + T[3])), (-T[1] * X * 
+              exp(T[2] * X + T[3]))/(1 + exp(T[2] * X + T[3]))^2, 
+              (-T[1] * exp(T[2] * X + T[3]))/(1 + exp(T[2] * X + 
+                T[3]))^2), nrow = 3, ncol = 1, byrow = F)
+          }      
+      f4=function(T,X) {
+            matrix(c(1/(1+exp(T[2]*X+T[3])),(-T[1]*X*exp(T[2]*X+T[3]))/(1+exp(T[2]*X+T[3]))^2,(-T[1]*exp(T[2]*X+T[3]))/(1+exp(T[2]*X+T[3]))			^2,1),nrow=4,ncol=1,byrow=F)
+      	  }
+ 
+      f <- function(T,X){
+      	if(np==2) {f2(T,X)} else if (np==3)
+      	{f3(T,X)} else {f4(T,X)}}
       M=upinfor(W,T,X)
       while(n<nit){
            x=seq(LB,UB,gr)
@@ -149,6 +201,7 @@ function(weight,P,dose,LB,UB,delta,r=10,grid=0.01,epsilon=.001,epsilon_w=10^-6)
       r=length(X)
       X=unique(X[(r-4):r])
       X=sort(X,decreasing=F)
+      if (length(X)==1) X=c(X,runif(1,LB,UB))
       p=1
       n=1
       it=1
@@ -191,7 +244,7 @@ function(weight,P,dose,LB,UB,delta,r=10,grid=0.01,epsilon=.001,epsilon_w=10^-6)
       for (i in 1:n1) 
            ds[i]=t(f(T,x1[i]))%*%inv%*%g(T,dt)%*%t(g(T,dt))%*%inv%*%f(T,x1[i])-BB
 
-      plot(x1,ds,cex=.3,main="Verify the c-optimal design for MED",ylab="Sensitivity function",xlab="Dose levels")
+      #plot(x1,ds,cex=.3,main="Verify the c-optimal design for MED",ylab="Sensitivity function",xlab="Dose levels")
 
       kk=length(dose)
       weight=weight[1:kk-1]
@@ -199,8 +252,13 @@ function(weight,P,dose,LB,UB,delta,r=10,grid=0.01,epsilon=.001,epsilon_w=10^-6)
       invs=ginv(upinfor(W,T,X))
       eff=(t(g(T,dt))%*%invs%*%g(T,dt))/(t(g(T,dt))%*%inv%*%g(T,dt))
       eff=as.vector(eff)
-      cat(format("c-optimal design for MED", width=80),"\n")
-      print(D)
-      cat(format("c-efficiency for MED", width=80),"\n")
-      print(eff)     
+  
+      R=new("PAR",fid="ceff2",LB=LB,UB=UB,grid=grid,ds=ds)
+      return(new("OPT",Par=R,Opt=D,Eff=eff))     
 }
+
+
+
+
+
+
